<html>
<head>
<meta http-equiv="refresh" content="{{ delay }}; url={{ url }}" />
</head>

<body>
<h1>Redirecting ...</h1>
<p>
    If you do not get redirected automatically in {{ delay }}s, feel free to click the link:
    <br>
    <br>
    <a href="{{ url }}">{{ url }}</a>
</p>

<h2>Warning</h2>
<p>
    Please note that the target site's content is neither controlled nor verified by us.
    Not even the link has been verified by us.
</p>
<p>
You use the internet on your own risk. If you break something, you own the pieces.
</p>
</body>
</html>
