{% extends "_layout.html" %}

{% block content %}
<div class="card">
    <div class="card-header">
        <h1>
            {{ heading }}
        </h1>
    </div>
    <div class="card-body">
        <div class="alert alert-danger">
            {{ body }}
        </div>
    </div>
</div>
{% endblock content %}
